from urllib import urlencode
from django.conf import settings
from django.http import HttpResponseBadRequest
from django.contrib.auth import REDIRECT_FIELD_NAME, authenticate, login as django_login
from django.core.urlresolvers import reverse
from django.shortcuts import render, redirect
import requests

from . import errors
from . import utils
from .utils import log
from .settings import oidc_settings
from .forms import OpenIDConnectForm
from .models import OpenIDProvider, get_default_provider, Nonce


def login_begin(request, template_name='oidc/login.html',
        form_class=OpenIDConnectForm,
        login_complete_view='oidc-complete',
        redirect_field_name=REDIRECT_FIELD_NAME):

    if _redirect_to_provider(request):
        return _redirect(request, login_complete_view, form_class, redirect_field_name)

    log.debug('Rendering login template at %s' % template_name)
    return render(request, template_name)


def _redirect(request, login_complete_view, form_class, redirect_field_name):
    provider = get_default_provider()

    if not provider:
        form = form_class(request.POST)

        if not form.is_valid():
            raise errors.MissingRedirectURL()

        provider = OpenIDProvider.discover(issuer=form.cleaned_data['issuer'])

    redirect_url = request.GET.get(redirect_field_name, settings.LOGIN_REDIRECT_URL)
    nonce = Nonce.generate(redirect_url, provider.issuer)
    request.session['oidc_state'] = nonce.state

    params = urlencode({
        'response_type': 'code',
        'scope': utils.scopes(),
        # 'redirect_uri': request.build_absolute_uri(reverse(login_complete_view)),
        'client_id': provider.client_id,
        'state': nonce.state
    })
    redirect_url = '%s?%s' % (provider.authorization_endpoint, params)

    log.debug('Redirecting to %s' % redirect_url)
    return redirect(redirect_url)


def login_complete(request, login_complete_view='oidc-complete',
        error_template_name='oidc/error.html'):

    if 'error' in request.GET:
        return render(request, error_template_name, {
            'error': request.GET['error']
        })

    if 'oidc_state' not in request.session:
        return redirect(settings.LOGIN_URL)

    if 'code' not in request.GET and 'state' not in request.GET:
        return HttpResponseBadRequest('Invalid request')

    if request.GET['state'] != request.session['oidc_state']:
        raise errors.ForbiddenAuthRequest()

    nonce = Nonce.objects.get(state=request.GET['state'])
    provider = nonce.provider
    log.debug('Login started from provider %s' % provider)

    params = {
        'grant_type': 'authorization_code',
        'code': request.GET['code'],
        'redirect_uri': request.build_absolute_uri(reverse(login_complete_view))
    }

    response = requests.post(provider.token_endpoint,
                             auth=provider.client_credentials,
                             params=params, verify=oidc_settings.VERIFY_SSL)

    if response.status_code != 200:
        raise errors.RequestError(provider.token_endpoint, response.status_code)

    log.debug('Token exchange done, proceeding authentication')
    credentials = response.json()
    credentials['provider'] = provider
    user = authenticate(credentials=credentials)
    django_login(request, user)

    return redirect(nonce.redirect_url)


def _redirect_to_provider(request):
    """Just a syntax sugar for login_begin. Returns True or False
    whether a request should be redirected to the provider or not.
    """

    has_default_provider = oidc_settings.DEFAULT_PROVIDER

    return (not oidc_settings.DISABLE_OIDC
            and (has_default_provider or request.method == 'POST'))



from django.conf.urls import patterns, url


urlpatterns = patterns('oidc_auth.views',
    url(r'^login/$', 'login_begin', name='oidc-login'),
    url(r'^complete/$', 'login_complete', name='oidc-complete'),
)



from .utils import log


class OpenIDConnectError(RuntimeError):
    def __init__(self, message=None):
        if not message:
            message = getattr(self, 'message', '')

        log.error(message)
        super(OpenIDConnectError, self).__init__(message)


class InvalidIdToken(OpenIDConnectError, ValueError):
    message = 'id_token MUST be signed'


class UnsuppportedSigningMethod(OpenIDConnectError, ValueError):
    def __init__(self, unsupported_method, supported_methods):
        message = 'Signing method %s not supported, options are (%s)' % (
                unsupported_method, ', '.join(supported_methods))

        super(UnsuppportedSigningMethod, self).__init__(message)


class RequestError(OpenIDConnectError):
    def __init__(self, url, status_code):
        message = 'GET %s returned %s status code (200 expected)' % (url, status_code)
        super(RequestError, self).__init__(message)


class InvalidUserInfo(OpenIDConnectError):
    message = 'The received sub does not match the value found in the ID token'


class ForbiddenAuthRequest(OpenIDConnectError):
    message = 'querystring state differs from state saved on session'


class MissingRedirectURL(OpenIDConnectError):
    message = "Missing URL for oidc redirect (maybe DEFAULT_ENDPOINT's missing on settings?)"



from django.contrib import admin

from .models import OpenIDProvider, OpenIDUser


class OpenIDProviderAdmin(admin.ModelAdmin):
    list_display = ('issuer', 'client_id')
    list_filter = ('signing_alg',)


class OpenIDUserAdmin(admin.ModelAdmin):
    list_display = ('sub', 'user')
    list_filter = ('issuer',)


admin.site.register(OpenIDProvider, OpenIDProviderAdmin)
admin.site.register(OpenIDUser, OpenIDUserAdmin)



from .utils import log, get_user_model
from .models import OpenIDUser


class OpenIDConnectBackend(object):
    supports_object_permissions = False
    supports_anonymous_user = True

    def get_user(self, user_id):
        UserModel = get_user_model()

        try:
            return UserModel.objects.get(pk=user_id)
        except UserModel.DoesNotExist:
            return None

    def authenticate(self, **kwargs):
        try:
            credentials = kwargs.get('credentials')
            if not credentials:
                return None

            provider = credentials['provider']
            id_token = provider.verify_id_token(credentials['id_token'])

            oidc_user = OpenIDUser.get_or_create(id_token,
                    credentials['access_token'],
                    credentials.get('refresh_token', ''),
                    provider)

            return oidc_user.user
        except Exception as e:
            log.error('Unexpected error on authentication: %s' % e)
            raise



from contextlib import contextmanager
from django.conf import settings


DEFAULTS = {
    'DISABLE_OIDC': False,
    'DEFAULT_PROVIDER': {},
    'SCOPES': ('openid', 'given_name', 'family_name', 'preferred_username', 'email'),
    'CLIENT_ID': None,
    'CLIENT_SECRET': None,
    'NONCE_LENGTH': 8,
    'VERIFY_SSL': True
}

USER_SETTINGS = getattr(settings, 'OIDC_AUTH', {})


class OIDCSettings(object):
    """Shamelessly copied from django-oauth-toolkit"""

    def __init__(self, user_settings, defaults):
        self.user_settings = user_settings
        self.defaults = defaults
        self.patched_settings = {}

    def __getattr__(self, attr):
        if attr not in self.defaults:
            raise AttributeError('Invalid oidc_auth setting: %s' % attr)

        if attr in self.patched_settings:
            val = self.patched_settings[attr]
        elif attr in self.user_settings:
            val = self.user_settings[attr]
        else:
            val = self.defaults[attr]

        return val

    @contextmanager
    def override(self, **kwargs):
        self.patched_settings = kwargs
        yield
        self.patched_settings = {}


oidc_settings = OIDCSettings(USER_SETTINGS, DEFAULTS)



import json
from base64 import b64decode as python_b64decode
import logging
from django.contrib import auth

from .settings import oidc_settings


def scopes():
    _scopes = set(oidc_settings.SCOPES)
    _scopes.update({'openid', 'email', 'preferred_username'})

    return ' '.join(_scopes)


def b64decode(token):
    token += ('=' * (len(token) % 4))
    decoded = python_b64decode(token)
    return json.loads(decoded)


def get_user_model():
    if hasattr(auth, 'get_user_model'):
        return auth.get_user_model()
    else:
        return auth.models.User


log = logging.getLogger('oidc_auth')
log.addHandler(logging.NullHandler())



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Nonce',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('issuer_url', models.URLField()),
                ('state', models.CharField(unique=True, max_length=255)),
                ('redirect_url', models.CharField(max_length=100)),
            ],
        ),
        migrations.CreateModel(
            name='OpenIDProvider',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('issuer', models.URLField(unique=True)),
                ('authorization_endpoint', models.URLField()),
                ('token_endpoint', models.URLField()),
                ('userinfo_endpoint', models.URLField()),
                ('jwks_uri', models.URLField(null=True, blank=True)),
                ('signing_alg', models.CharField(default=b'HS256', max_length=5, choices=[(b'RS256', b'RS256'), (b'HS256', b'HS256')])),
                ('client_id', models.CharField(max_length=255)),
                ('client_secret', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='OpenIDUser',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('sub', models.CharField(unique=True, max_length=255)),
                ('access_token', models.CharField(max_length=255)),
                ('refresh_token', models.CharField(max_length=255)),
                ('issuer', models.ForeignKey(to='oidc_auth.OpenIDProvider')),
                ('user', models.OneToOneField(related_name='oidc_account', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



import string
import random
import json
from urlparse import urljoin
import requests
from django.db import models, IntegrityError
from django.conf import settings
from jwkest.jwk import load_jwks_from_url
from jwkest.jws import JWS
from jwkest.jwk import SYMKey

from . import errors
from .settings import oidc_settings
from .utils import log, b64decode, get_user_model


class Nonce(models.Model):
    issuer_url = models.URLField()
    state = models.CharField(max_length=255, unique=True)
    redirect_url = models.CharField(max_length=100)

    def __unicode__(self):
        return '%s' % self.state

    def __init__(self, *args, **kwargs):
        self._provider = None
        super(Nonce, self).__init__(*args, **kwargs)

    @classmethod
    def generate(cls, redirect_url, issuer, length=oidc_settings.NONCE_LENGTH):
        """This method generates and returns a nonce, an unique generated
        string. If the maximum of retries is exceeded, it returns None.
        """
        CHARS = string.letters + string.digits

        for i in range(5):
            _hash = ''.join(random.choice(CHARS) for n in range(length))

            try:
                log.debug('Attempt %s to save nonce %s to issuer %s' % (i+1,
                    _hash, issuer))
                return cls.objects.create(issuer_url=issuer, state=_hash,
                        redirect_url=redirect_url)
            except IntegrityError:
                pass

        log.debug('Maximum of retries to create a nonce to issuer %s '
                  'exceeded! Max: 5' % issuer)

    @property
    def provider(self):
        """This method will fetch the related provider from the database
        and cache it inside an object var.
        """
        provider = self._provider

        if not provider:
            provider = OpenIDProvider.objects.get(issuer=self.issuer_url)

        return provider


class OpenIDProvider(models.Model):
    RS256 = 'RS256'
    HS256 = 'HS256'
    SIGNING_ALGS = (
        (RS256, 'RS256'),
        (HS256, 'HS256'),
    )

    issuer = models.URLField(unique=True)
    authorization_endpoint = models.URLField()
    token_endpoint = models.URLField()
    userinfo_endpoint = models.URLField()
    jwks_uri = models.URLField(null=True, blank=True)
    signing_alg = models.CharField(max_length=5, choices=SIGNING_ALGS, default=HS256)

    client_id = models.CharField(max_length=255)
    client_secret = models.CharField(max_length=255)

    def __unicode__(self):
        return self.issuer

    @classmethod
    def discover(cls, issuer='', credentials={}, save=True):
        """Returns a known OIDC Endpoint. If it doesn't exist in the database,
        then it'll fetch its data according to OpenID Connect Discovery spec.
        """
        if not (issuer or credentials):
            raise ValueError('You should provide either an issuer or credentials')

        if not issuer:
            issuer = cls._get_issuer(credentials['id_token'])

        try:
            provider = cls.objects.get(issuer=issuer)
            log.debug('Provider %s already discovered' % issuer)
            return provider
        except cls.DoesNotExist:
            pass

        log.debug('Provider %s not discovered yet, proceeding discovery' % issuer)
        discover_endpoint = urljoin(issuer, '.well-known/openid-configuration')
        response = requests.get(discover_endpoint, verify=oidc_settings.VERIFY_SSL)

        if response.status_code != 200:
            raise errors.RequestError(discover_endpoint, response.status_code)

        configs = response.json()
        provider = cls()

        provider.issuer = configs['issuer']
        provider.authorization_endpoint = configs['authorization_endpoint']
        provider.token_endpoint = configs['token_endpoint']
        provider.userinfo_endpoint = configs['userinfo_endpoint']
        provider.jwks_uri = configs['jwks_uri']

        if save:
            provider.save()

        log.debug('Provider %s succesfully discovered' % issuer)
        return provider

    @property
    def client_credentials(self):
        return self.client_id, self.client_secret

    @property
    def signing_keys(self):
        if self.signing_alg == self.RS256:
            # TODO perform caching, OBVIOUS
            return load_jwks_from_url(self.jwks_uri)

        return [SYMKey(key=str(self.client_secret))]

    def verify_id_token(self, token):
        log.debug('Verifying token %s' % token)
        header, claims, signature = token.split('.')
        header = b64decode(header)
        claims = b64decode(claims)

        if not signature:
            raise errors.InvalidIdToken()

        if header['alg'] not in ['HS256', 'RS256']:
            raise errors.UnsuppportedSigningMethod(header['alg'], ['HS256', 'RS256'])

        id_token = JWS().verify_compact(token, self.signing_keys)
        log.debug('Token verified, %s' % id_token)
        return json.loads(id_token)

    @staticmethod
    def _get_issuer(token):
        """Parses an id_token and returns its issuer.

        An id_token is a string containing 3 b64-encrypted hashes,
        joined by a dot, like:

            <header>.<claims>.<signature>

        We only need to parse the claims, which contains the 'iss' field
        we're looking for.
        """
        _, claims, _ = token.split('.')

        return b64decode(claims)['iss']


def get_default_provider():
    args = oidc_settings.DEFAULT_PROVIDER

    if not args:
        return

    issuer = args.get('issuer')
    provider, created = OpenIDProvider.objects.get_or_create(issuer=issuer, defaults=args)

    if created:
        return provider

    # Test if the object is up-to-date
    should_update = False
    fields = ['authorization_endpoint', 'token_endpoint',
              'userinfo_endpoint', 'client_id', 'client_secret']

    for field in fields:
        if getattr(provider, field) != args[field]:
            should_update = True
            setattr(provider, field, args[field])

    if should_update:
        provider.save()

    return provider


class OpenIDUser(models.Model):
    sub = models.CharField(max_length=255, unique=True)
    issuer = models.ForeignKey(OpenIDProvider)
    user = models.OneToOneField(getattr(settings, 'AUTH_USER_MODEL', 'auth.User'),
            related_name='oidc_account')

    access_token = models.CharField(max_length=255)
    refresh_token = models.CharField(max_length=255)

    def __unicode__(self):
        return '%s: %s' % (self.sub, self.user)

    @classmethod
    def get_or_create(cls, id_token, access_token, refresh_token, provider):
        UserModel = get_user_model()

        try:
            oidc_acc = cls.objects.get(sub=id_token['sub'])

            # Updating with new tokens
            oidc_acc.access_token = access_token
            oidc_acc.refresh_token = refresh_token
            oidc_acc.save()

            log.debug('OpenIDUser found, sub %s' % oidc_acc.sub)
            return oidc_acc
        except cls.DoesNotExist:
            log.debug("OpenIDUser for sub %s not found, so it'll be created" % id_token['sub'])

        # Find an existing User locally or create a new one
        try:
            user = UserModel.objects.get(username__iexact=id_token['sub'])
            log.debug('Found user with username %s locally' % id_token['sub'])
        except UserModel.MultipleObjectsReturned:
            user = UserModel.objects.filter(username__iexact=id_token['sub'])[0]
            log.warn('Multiple users found with username %s! First match will be selected' % id_token['sub'])
        except UserModel.DoesNotExist:
            log.debug('User with username %s not found locally, '
                      'so it will be created' % id_token['sub'])

            user = UserModel()

        # Always update user's local data
        claims = cls._get_userinfo(provider, id_token['sub'],
                                   access_token, refresh_token)

        user.username   = claims['preferred_username']
        user.email      = claims['email']
        user.first_name = claims['given_name']
        user.last_name  = claims['family_name']
        user.is_superuser = claims.get('is_superuser', False)
        user.is_staff = claims.get('is_staff', False)

        user.set_unusable_password()
        user.save()

        # Avoid duplicate user key
        try:
            oidc_acc = cls.objects.get(user=user)

            oidc_acc.sub= id_token['sub']
            oidc_acc.access_token = access_token
            oidc_acc.refresh_token = refresh_token
            oidc_acc.save()

            log.debug('OpenIDUser found, sub %s' % oidc_acc.sub)
            return oidc_acc
        except cls.DoesNotExist:
            log.debug("OpenIDUser for sub %s not found, so it'll be created" % id_token['sub'])

        return cls.objects.create(sub=id_token['sub'], issuer=provider,
                user=user, access_token=access_token, refresh_token=refresh_token)

    @classmethod
    def _get_userinfo(cls, provider, sub, access_token, refresh_token):
        # TODO encapsulate this?
        log.debug('Requesting userinfo in %s. sub: %s, access_token: %s' % (
            provider.userinfo_endpoint, sub, access_token))
        response = requests.get(provider.userinfo_endpoint, headers={
            'Authorization': 'Bearer %s' % access_token
        }, verify=oidc_settings.VERIFY_SSL)

        if response.status_code != 200:
            raise errors.RequestError(provider.userinfo_endpoint, response.status_code)

        claims = response.json()

        if claims['sub'] != sub:
            raise errors.InvalidUserInfo()

        name = '%s %s' % (claims['given_name'], claims['family_name'])
        log.debug('t userinfo of sub: %s -> name: %s, preferred_username: %s, email: %s' % (sub,
            name, claims['preferred_username'], claims['email']))

        return claims



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages


setup(
    name='django-oidc-auth',
    version='0.0.11',
    description='OpenID Connect client for Django applications',
    long_description='WIP',
    author='Lucas S. Magalhães',
    author_email='lucas.sampaio@intelie.com.br',
    packages=find_packages(exclude=['*.tests']),
    include_package_data=True,
    install_requires=[
        'Django>=1.5',
        'South==1.0.2',
        'pyjwkest==0.6.2',
        'requests',
    ],
    zip_safe=True
)



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Nonce'
        db.create_table(u'oidc_auth_nonce', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('issuer_url', self.gf('django.db.models.fields.URLField')(max_length=200)),
            ('state', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255)),
            ('redirect_url', self.gf('django.db.models.fields.CharField')(max_length=100)),
        ))
        db.send_create_signal(u'oidc_auth', ['Nonce'])

        # Adding model 'OpenIDProvider'
        db.create_table(u'oidc_auth_openidprovider', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('issuer', self.gf('django.db.models.fields.URLField')(unique=True, max_length=200)),
            ('authorization_endpoint', self.gf('django.db.models.fields.URLField')(max_length=200)),
            ('token_endpoint', self.gf('django.db.models.fields.URLField')(max_length=200)),
            ('userinfo_endpoint', self.gf('django.db.models.fields.URLField')(max_length=200)),
            ('jwks_uri', self.gf('django.db.models.fields.URLField')(max_length=200, null=True, blank=True)),
            ('signing_alg', self.gf('django.db.models.fields.CharField')(default='HS256', max_length=5)),
            ('client_id', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('client_secret', self.gf('django.db.models.fields.CharField')(max_length=255)),
        ))
        db.send_create_signal(u'oidc_auth', ['OpenIDProvider'])

        # Adding model 'OpenIDUser'
        db.create_table(u'oidc_auth_openiduser', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('sub', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255)),
            ('issuer', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['oidc_auth.OpenIDProvider'])),
            ('user', self.gf('django.db.models.fields.related.OneToOneField')(related_name='oidc_account', unique=True, to=orm['auth.User'])),
            ('access_token', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('refresh_token', self.gf('django.db.models.fields.CharField')(max_length=255)),
        ))
        db.send_create_signal(u'oidc_auth', ['OpenIDUser'])


    def backwards(self, orm):
        # Deleting model 'Nonce'
        db.delete_table(u'oidc_auth_nonce')

        # Deleting model 'OpenIDProvider'
        db.delete_table(u'oidc_auth_openidprovider')

        # Deleting model 'OpenIDUser'
        db.delete_table(u'oidc_auth_openiduser')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Group']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'oidc_auth.nonce': {
            'Meta': {'object_name': 'Nonce'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'issuer_url': ('django.db.models.fields.URLField', [], {'max_length': '200'}),
            'redirect_url': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'state': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'})
        },
        u'oidc_auth.openidprovider': {
            'Meta': {'object_name': 'OpenIDProvider'},
            'authorization_endpoint': ('django.db.models.fields.URLField', [], {'max_length': '200'}),
            'client_id': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'client_secret': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'issuer': ('django.db.models.fields.URLField', [], {'unique': 'True', 'max_length': '200'}),
            'jwks_uri': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'signing_alg': ('django.db.models.fields.CharField', [], {'default': "'HS256'", 'max_length': '5'}),
            'token_endpoint': ('django.db.models.fields.URLField', [], {'max_length': '200'}),
            'userinfo_endpoint': ('django.db.models.fields.URLField', [], {'max_length': '200'})
        },
        u'oidc_auth.openiduser': {
            'Meta': {'object_name': 'OpenIDUser'},
            'access_token': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'issuer': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['oidc_auth.OpenIDProvider']"}),
            'refresh_token': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'sub': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'user': ('django.db.models.fields.related.OneToOneField', [], {'related_name': "'oidc_account'", 'unique': 'True', 'to': u"orm['auth.User']"})
        }
    }

    complete_apps = ['oidc_auth']